/**
 * Highly Divisible Triangular Number.
 *
 * This program finds the first triangular number with over 500 divisors.
 */
public class Problem012 implements EulerSolution {
	public static void main(String[] args) { 
		Problem012 p = new Problem012();
		System.out.println(p.run());
	}

	public String run() {
		return new Long( findTriangularNumber(500) ).toString();
	}

	/** Find a triangular number with over k divisors. */
	public static long findTriangularNumber(long k) { 
		int divisors = 0;

		// Note: the smallest number with 500 factors 
		// has approx log_2( 500 ) prime factors.
		// That's 8.9, or rounding down, 8. 
		// A number with 2^9 prime divisors has 512 factors.
		//
		// Next, we know the value of n is proportional to the 
		// square root of the expression, so we approximate
		// our starting point by picking a number
		// that will start at or below the right neighborhood
		// and work our way up.
		// 
		long n = (long)(Math.sqrt(2*3*5*7*11*13*17*19));
		long tri = 0;

		// We want first number with OVER k divisors
		while(divisors <= k) { 
			n++;
			tri = EulerLib.triangleLong(n);
			divisors = countDivisors(tri);
			//System.out.printf("%d\t%d\t%d\n",n,EulerLib.triangleLong(n),divisors);
		}

		return tri;
	}

	/** Count the number of divisors of n. */
	public static int countDivisors(long n) { 
		long max = (long)(Math.sqrt(n));
		int divisors = 0;

		// one and itself:
		divisors += 2;
		// unless you're 2
		if(n==2) divisors--;

		// double-count up to sqrt(n)
		for(long i=2; i<=max; i++) { 
			if(EulerLib.gcfLong(i,n)==i) { 
				// count this divisor, 
				// plus its >sqrt(n) complement
				divisors += 2;
			}
		}
		return divisors;
	}

}
