import java.util.*;

/** 
 * Count the number of letters that are used when writing out 
 * each number from 1 to 1000 in words.
 *
 * This is an interval problem: 
 * each interval has particular rules for construction.
 * Some are special cases.
 *
 * This will work recursively.
 * Wow... this is kind of a monstrous problem, when you think about it,
 * because it isn't about math as much as it is about translating 
 * baffling linguistic rules into computer code.
 *
 */
public class Problem017 implements EulerSolution {

	public static Map<Integer,String> m = new TreeMap<Integer,String>();

	public String run() {
		return countLetters(1000).toString();
	}


	/** Count the number of letters in every number up to n spelled out. */
	public static Integer countLetters(int n) { 
		int cumsum = 0;
		for(int j=1; j<=n; j++) {
			String e = englishify(j);
			cumsum += countAlphas(e);
		}
		return new Integer(cumsum);
	}


	/** Count the number of letters in this digit's English representation. */ 
	public static int countNumber(int n) { 
		String e = englishify(n);
		return countAlphas(e);
	}


	public static int countAlphas(String s) { 
		int count = 0;
		for(int j=0; j<s.length(); j++) { 
			if(Character.isAlphabetic(s.charAt(j))) {
				count++;
			}
		}
		return count;
	}


	/** Englishify the digits.
	 * */
	public static String englishify( int n ) {
		if(n > 1000 || n < 1) { 
			throw new IllegalArgumentException("Error: more than 2 digits/no digits.");
		}
		
		// Deal with the number in intervals.
		// This is the only sane way to do this.
		char[] digits = Integer.toString(n).toCharArray(); 
		if(digits.length>4 || digits.length<1 ) { 
			throw new IllegalArgumentException("Error: more than 2 digits/no digits.");
		}
		int ndigits = digits.length;
		

		if( n==1000 ) { 
			return thousand();

		} else if( ndigits==3 ) { 
			return threeDigit(digits,n);
		
		} else if( ndigits==2 ) {
			return twoDigit(digits,n);

		} else if( ndigits==1 ) { 
			return oneDigit(digits,n);

		} else { 
			return "";
		}
	}

	public static String threeDigit(char[] digits, int n) { 
		StringBuffer sb = new StringBuffer();
		Map<Integer,String> m = getSpecials();
		if(n%100==0) { 

			// turn left most digit into word, add hundred
			// 100 -> one hundred
			int lead_digit = Character.getNumericValue(digits[0]);
			sb.append(m.get(lead_digit));
			sb.append(" hundred");

		} else {

			// turn left most digit into word, add "hundred and"
			int lead_digit = Character.getNumericValue(digits[0]);
			sb.append(m.get(lead_digit));
			sb.append(" hundred and ");

			// remove the digit we just dealt with, 
			// in the char[] and the integer,
			// and pass it on to the two digit method 
			char[] shortDigits = new char[digits.length-1];
			for(int j=0; j<digits.length-1; j++) { 
				shortDigits[j] = digits[j+1];
			}
			sb.append( twoDigit(shortDigits,(n%100)) );

		}
		return sb.toString();
	}

	public static String twoDigit(char[] digits, int n) { 
		StringBuffer sb = new StringBuffer();
		Map<Integer,String> m = getSpecials();
		if(n%10==0) { 
			sb.append(m.get(n));

		} else if( n>10 && n<20) { 
			// Don't use the tens-ones approach, just look up n directly
			sb.append(m.get(n));

		} else {

			// Turn the tens place into the tens word
			int lead_digit = Character.getNumericValue(digits[0]);
			if(lead_digit>0) { 
				sb.append(m.get(lead_digit*10));
				sb.append(" ");
			}

			// Remove the digit we just dealt with,
			// and pass the rest on to the one digit method.
			char[] shortDigits = new char[digits.length-1];
			for(int j=0; j<digits.length-1; j++) {
				shortDigits[j] = digits[j+1];
			}
			sb.append( oneDigit(shortDigits,(n%10)) );

		}
		return sb.toString();
	}

	public static String oneDigit(char[] digits, int n) { 
		StringBuffer sb = new StringBuffer();
		Map<Integer,String> m = getSpecials();
		if(n>0 && n<10) { 
			sb.append(m.get(n));
		}
		return sb.toString();
	}

	public static String thousand() {
		// Next steps:
		// If we made this work for hundreds of thousands, this would go in a map.
		return "one thousand";
	}









	/** Get special cases: multiples of 10 */
	public static Map<Integer,String> getSpecials() {
		// 1 to 10
		m.put(1,"one");
		m.put(2,"two");
		m.put(3,"three");
		m.put(4,"four");
		m.put(5,"five");
		m.put(6,"six");
		m.put(7,"seven");
		m.put(8,"eight");
		m.put(9,"nine");

		// teens
		m.put(11,"eleven");
		m.put(12,"twelve");
		m.put(13,"thirteen");
		m.put(14,"fourteen");
		m.put(15,"fifteen");
		m.put(16,"sixteen");
		m.put(17,"seventeen");
		m.put(18,"eighteen");
		m.put(19,"nineteen");

		// multiples of 10
		m.put(10,"ten");
		m.put(20,"twenty");
		m.put(30,"thirty");
		m.put(40,"forty");
		m.put(50,"fifty");
		m.put(60,"sixty");
		m.put(70,"seventy");
		m.put(80,"eighty");
		m.put(90,"ninety");

		return m;
	}

}
