import java.math.BigInteger;

public class Problem027 {
    public static void main(String[] args) {
        long startTime = System.currentTimeMillis();
        int[] result = findBestQuadratic();
        long endTime = System.currentTimeMillis();
        
        int bestA = result[0];
        int bestB = result[1];
        int maxConsecutivePrimes = result[2];
        
        System.out.println("Best quadratic formula: n² + " + bestA + "n + " + bestB);
        System.out.println("Maximum consecutive primes: " + maxConsecutivePrimes);
        System.out.println("Product of coefficients: " + (bestA * bestB));
        System.out.println("Execution time: " + (endTime - startTime) + " ms");
    }
    
    public static int[] findBestQuadratic() {
        int bestA = 0;
        int bestB = 0;
        int maxConsecutivePrimes = 0;
        
        // We know b must be prime since when n=0, formula = b
        // So we'll only test prime values for b
        for (int b = -999; b <= 1000; b++) {
            if (!isPrime(Math.abs(b))) continue;
            
            for (int a = -999; a < 1000; a++) {
                int n = 0;
                while (isPrime(Math.abs(n * n + a * n + b))) {
                    n++;
                }
                
                if (n > maxConsecutivePrimes) {
                    maxConsecutivePrimes = n;
                    bestA = a;
                    bestB = b;
                }
            }
        }
        
        return new int[]{bestA, bestB, maxConsecutivePrimes};
    }
    
    public static boolean isPrime(int num) {
        if (num <= 1) return false;
        if (num <= 3) return true;
        if (num % 2 == 0 || num % 3 == 0) return false;
        
        int sqrtNum = (int) Math.sqrt(num) + 1;
        for (int i = 5; i <= sqrtNum; i += 6) {
            if (num % i == 0 || num % (i + 2) == 0) return false;
        }
        
        return true;
    }
    
    // For larger numbers, we can use a more robust prime check
    public static boolean isPrimeBigInteger(int num) {
        if (num <= 1) return false;
        if (num <= 3) return true;
        
        BigInteger bigInt = BigInteger.valueOf(num);
        return bigInt.isProbablePrime(10); // certainty parameter of 10 is good enough
    }
}
