import java.math.BigInteger;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.TimeUnit; // Optional: for more readable time output

/**
 * Calculates the number of distinct terms of the form a^b
 * for 2 <= a <= 100 and 2 <= b <= 100.
 * This implementation runs on a single core/thread.
 */
public class Problem029 {

    // Define the inclusive range for base 'a'
    private static final int MIN_A = 2;
    private static final int MAX_A = 100;

    // Define the inclusive range for exponent 'b'
    private static final int MIN_B = 2;
    private static final int MAX_B = 100;

    public static void main(String[] args) {

        System.out.println("Calculating distinct powers for a^b where "
                + MIN_A + " <= a <= " + MAX_A + " and "
                + MIN_B + " <= b <= " + MAX_B + "...");
        System.out.println("Running on a single thread...");

        // Start timing
        long startTime = System.nanoTime();

        // Use a HashSet to store distinct BigInteger results.
        // HashSet ensures that each element added is unique.
        Set<BigInteger> distinctTerms = new HashSet<>();

        // Iterate through all possible values of base 'a'
        for (int a = MIN_A; a <= MAX_A; ++a) {
            // Convert the integer base 'a' to BigInteger.
            // This is necessary because the results of a^b can exceed
            // the limits of standard primitive types like 'long'.
            BigInteger base = BigInteger.valueOf(a);

            // Iterate through all possible values of exponent 'b'
            for (int b = MIN_B; b <= MAX_B; ++b) {
                // Calculate a^b using BigInteger's pow() method.
                // The exponent 'b' is within the 'int' range, which is
                // required by the pow() method.
                BigInteger result = base.pow(b);

                // Add the calculated BigInteger result to the HashSet.
                // If 'result' is already in the set, the add() method
                // does nothing and returns false.
                distinctTerms.add(result);
            }
        }

        // The size of the set directly corresponds to the number of
        // distinct terms generated.
        int count = distinctTerms.size();

        // Stop timing
        long endTime = System.nanoTime();

        // Calculate and format duration
        long durationNanos = endTime - startTime;
        // Convert nanoseconds to milliseconds for readability
        long durationMillis = TimeUnit.NANOSECONDS.toMillis(durationNanos);

        System.out.println("Calculation complete.");
        System.out.println("Number of distinct terms generated: " + count);
        System.out.println("Time taken: " + durationMillis + " ms (" + durationNanos + " ns)");
    }
}
