import java.util.HashSet;
import java.util.Set;

public class Problem032 {
    public static void main(String[] args) {
        Set<Integer> pandigitalProducts = findPandigitalProducts();
        
        // Calculate the sum of all pandigital products
        int sum = 0;
        for (int product : pandigitalProducts) {
            sum += product;
        }
        
        System.out.println("Found " + pandigitalProducts.size() + " pandigital products.");
        System.out.println("The sum of all pandigital products is: " + sum);

        // // Debug
        // for (int product : pandigitalProducts) {
        //     System.out.println(product);
        // }
    }
    
    public static Set<Integer> findPandigitalProducts() {
        Set<Integer> pandigitalProducts = new HashSet<>();
        
        // The product must be at least 4 digits (1000) and at most 5 digits (99999)
        int MIN_PRODUCT = 1000;
        int MAX_PRODUCT = 10000;
        for (int product = MIN_PRODUCT; product < MAX_PRODUCT; product++) {
            // Find all factors and check if they form a pandigital expression
            checkFactors(product, pandigitalProducts);
        }
        
        return pandigitalProducts;
    }
    
    private static void checkFactors(int product, Set<Integer> pandigitalProducts) {
        // We'll check possible factors up to the square root of the product
        int upperLimit = (int) Math.sqrt(product);
        
        for (int factor1 = 2; factor1 <= upperLimit; factor1++) {
            // If factor1 is a divisor of product
            if (product % factor1 == 0) {
                int factor2 = product / factor1;
                
                // Check if the expression factor1 × factor2 = product is pandigital
                if (isPandigitalExpression(factor1, factor2, product)) {
                    pandigitalProducts.add(product);
                    // We found a valid expression, no need to check more factors for this product
                    return;
                }
            }
        }
    }
    
    public static boolean isPandigitalExpression(int factor1, int factor2, int product) {
        // Convert the entire expression to a string
        String expression = factor1 + "" + factor2 + "" + product;
        
        // The expression should be exactly 9 characters long
        if (expression.length() != 9) {
            return false;
        }
        
        // Check if all digits 1-9 are present exactly once
        boolean[] digits = new boolean[10]; // digits[0] will be ignored
        
        for (char c : expression.toCharArray()) {
            int digit = c - '0';
            
            // Check if the digit is between 1-9
            if (digit < 1 || digit > 9) {
                return false;
            }
            
            // Check if the digit has already been seen
            if (digits[digit]) {
                return false;
            }
            
            digits[digit] = true;
        }
        
        // If we got here, all digits 1-9 are present exactly once
        return true;
    }
}
