import java.util.ArrayList;
import java.util.List;

public class Problem033 {
    
    static class Fraction {
        int numerator;
        int denominator;
        
        public Fraction(int numerator, int denominator) {
            this.numerator = numerator;
            this.denominator = denominator;
        }
        
        public double getValue() {
            return (double) numerator / denominator;
        }
        
        public String toString() {
            return numerator + "/" + denominator;
        }
        
        // Returns the greatest common divisor of two numbers
        private int gcd(int a, int b) {
            if (b == 0) return a;
            return gcd(b, a % b);
        }
        
        // Checks if the fraction is in the simplest form
        public boolean isSimplestForm() {
            return gcd(numerator, denominator) == 1;
        }
        
        // Returns true if this is a trivial case (has a common digit of 0)
        public boolean isTrivial() {
            int num1 = numerator / 10;
            int num2 = numerator % 10;
            int denom1 = denominator / 10;
            int denom2 = denominator % 10;
            
            return (num1 == 0 && denom1 == 0) || (num2 == 0 && denom2 == 0);
        }
    }
    
    public static void main(String[] args) {
        List<Fraction> validFractions = findDigitCancellationFractions();
        
        System.out.println("The four non-trivial digit cancellation fractions are:");
        for (Fraction f : validFractions) {
            System.out.println(f);
        }
        
        // Calculate product
        int productNum = 1;
        int productDenom = 1;
        
        for (Fraction f : validFractions) {
            productNum *= f.numerator;
            productDenom *= f.denominator;
        }
        
        // Simplify the product
        int gcd = gcd(productNum, productDenom);
        productNum /= gcd;
        productDenom /= gcd;
        
        System.out.println("\nThe product of these fractions is: " + productNum + "/" + productDenom);
        System.out.println("So the denominator of the product is: " + productDenom);
    }
    
    private static int gcd(int a, int b) {
        if (b == 0) return a;
        return gcd(b, a % b);
    }
    
    private static List<Fraction> findDigitCancellationFractions() {
        List<Fraction> results = new ArrayList<>();
        
        // Loop through all possible 2-digit fractions less than 1
        for (int denominator = 10; denominator <= 99; denominator++) {
            // Only need to check numerators less than denominator
            for (int numerator = 10; numerator < denominator; numerator++) {
                // Extract digits
                int numTens = numerator / 10;
                int numOnes = numerator % 10;
                int denomTens = denominator / 10;
                int denomOnes = denominator % 10;
                
                // Check for shared digits
                if (numTens == denomTens || numTens == denomOnes || 
                    numOnes == denomTens || numOnes == denomOnes) {
                    
                    Fraction original = new Fraction(numerator, denominator);
                    
                    double originalValue = (double) numerator / denominator;
                    
                    // Check all four possible digit cancellations
                    // Case 1: Cancel tens digit of numerator with tens digit of denominator
                    if (numTens == denomTens && numTens != 0) {
                        double cancelledValue = (double) numOnes / denomOnes;
                        if (Math.abs(originalValue - cancelledValue) < 0.0000001) {
                            if (!original.isTrivial()) {
                                results.add(original);
                                continue;
                            }
                        }
                    }
                    
                    // Case 2: Cancel tens digit of numerator with ones digit of denominator
                    if (numTens == denomOnes && numTens != 0) {
                        double cancelledValue = (double) numOnes / denomTens;
                        if (Math.abs(originalValue - cancelledValue) < 0.0000001) {
                            if (!original.isTrivial()) {
                                results.add(original);
                                continue;
                            }
                        }
                    }
                    
                    // Case 3: Cancel ones digit of numerator with tens digit of denominator
                    if (numOnes == denomTens && numOnes != 0) {
                        double cancelledValue = (double) numTens / denomOnes;
                        if (Math.abs(originalValue - cancelledValue) < 0.0000001) {
                            if (!original.isTrivial()) {
                                results.add(original);
                                continue;
                            }
                        }
                    }
                    
                    // Case 4: Cancel ones digit of numerator with ones digit of denominator
                    if (numOnes == denomOnes && numOnes != 0) {
                        double cancelledValue = (double) numTens / denomTens;
                        if (Math.abs(originalValue - cancelledValue) < 0.0000001) {
                            if (!original.isTrivial()) {
                                results.add(original);
                                continue;
                            }
                        }
                    }
                }
            }
        }
        
        return results;
    }
}
