import java.math.BigInteger;

/** Counting Lychrel Numbers Below 10,000.
 *
 * This counts the number of Lychrel numbers,
 * which are numbers that never produce a 
 * palindrome when added to the reverse of 
 * its own digits.
 */
public class Problem055 implements EulerSolution {

	public String run() { 
		int count = 0;
		for(int i=1; i<10000; i++){ 
			if(isLychrel(i)) { 
				count++;
			}
		}
		return Integer.toString(count);
	}

	/** Boolean: is n a Lychresl number (never reaches a palindrome)? */
	public static boolean isLychrel(int n) { 
		int it = countIterations(n);
		return it==51;
	}

	/** Count the number of iterations (steps) it takes to get to a palindrome.
	 * One step: 47 + 74 = 121
	 *
	 * Three steps: 
	 * 349 + 943 = 1292
	 * 1292 + 2921 = 4213
	 * 4213 + 3124 = 7337
	 */
	public static int countIterations(int n) {
		if(n>10000) { 
			throw new IllegalArgumentException("This method operates on the assumption that n < 10,000.");
		}

		// Even Palindromes can be Lychrel numbers, 
		// so do at least one step.
		//
		// steps is what we return.
		int steps = 0;
		final int MAXSTEPS = 50;

		BigInteger b = new BigInteger(Integer.toString(n));
		BigInteger r = new BigInteger(StringLib.reverse(b.toString()));
		BigInteger s = b.add(r);
		steps++;

		while( !StringLib.isPalindrome(s.toString()) && steps <= MAXSTEPS ) { 
			b = s;
			r = new BigInteger(StringLib.reverse(b.toString()));
			s = b.add(r);
			steps++;
		}

		// This little twist of logic ensures that if there are exactly 50 iterations,
		// we don't accidentally classify it as a Lychrel number.
		if(!StringLib.isPalindrome(s.toString())) {
			return MAXSTEPS+1;
		} else {
			return steps;
		}
	}

}
