import java.math.BigInteger;

/** 
 * Find the number of terms in the first 1,000 iterations
 * of the continued fraction of sqrt(2) whose denominator
 * has more digits than its numerator.
 *
 * This utilizes the recurrence relation for the nth iteration,
 * a(1) = 3,  b(1) = 2
 *
 * a_i = a_{i-1} + 2 b_{i-1}
 *
 * b_i = a_{i-1} + b_{i-1}
 *
 * I actually implemented this in Excel, to begin with, just cuz,
 * but these numbers get REALLY huge, REALLY fast.
 *
 * Note, from 2 weeks later: my adventure in 
 * continued fractions makes the approach below 
 * seem... primitive.
 */
public class Problem057 implements EulerSolution { 
	public static final BigInteger TWO = new BigInteger("2");

	public String run() {
		BigInteger aim1 = new BigInteger("3");
		BigInteger bim1 = new BigInteger("2");
		BigInteger ai, bi;

		int nterms = 1000;
		int noverflows = 0;
		for(int i=0; i<nterms; i++) { 
			ai = aim1.add( bim1.multiply(TWO) );
			bi = aim1.add( bim1 );
			if( ai.toString().length() > bi.toString().length() ) { 
				noverflows++;
			}
			aim1 = ai;
			bim1 = bi;
		}
		return Integer.toString(noverflows);
	}
}

