import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


public class Problem059 {
    public static void main(String[] args) {
        try {
            // Load the ciphertext
            String content = new String(Files.readAllBytes(Paths.get("p059_input.txt")));
            
            // Parse the comma-separated integers
            List<Integer> cipher = Arrays.stream(content.trim().split(","))
                                         .map(Integer::parseInt)
                                         .collect(Collectors.toList());
            
            // Try all possible 3-letter lowercase keys
            for (int k1 = 97; k1 <= 122; k1++) {  // ASCII 'a' to 'z'
                for (int k2 = 97; k2 <= 122; k2++) {
                    for (int k3 = 97; k3 <= 122; k3++) {
                        int[] keys = {k1, k2, k3};
                        List<Integer> plain = new ArrayList<>();
                        
                        // Decrypt
                        for (int i = 0; i < cipher.size(); i++) {
                            int k = keys[i % 3];
                            plain.add(cipher.get(i) ^ k);
                        }
                        
                        // Convert to string and check if it looks like English
                        StringBuilder plaintextBuilder = new StringBuilder();
                        for (int j : plain) {
                            plaintextBuilder.append((char) j);
                        }
                        String plaintext = plaintextBuilder.toString();
                        
                        if (isEnglishLike(plaintext)) {
                            System.out.println("Possible key: " + (char)k1 + (char)k2 + (char)k3);
                            System.out.println("Decrypted text sample: " + 
                                plaintext.substring(0, Math.min(50, plaintext.length())) + "...");
                            
                            // Sum of ASCII values
                            int sum = plain.stream().mapToInt(Integer::intValue).sum();
                            System.out.println("Sum of ASCII values: " + sum);
                            
                            // Uncomment to see full plaintext
                            // System.out.println("Full plaintext: " + plaintext);
                        }
                    }
                }
            }
            
        } catch (IOException e) {
            System.err.println("Error reading file: " + e.getMessage());
        }
    }
    
    /**
     * Check if text appears to be written in English based on character frequency analysis
     * @param text The text to analyze
     * @return true if the text appears to be English
     */
    private static boolean isEnglishLike(String text) {
        // Simple check for English-likeness: Count spaces and common letters
        double spaceRatio = (double) countOccurrences(text, ' ') / text.length();
        
        String commonLetters = "etaoinshrdlu";
        int commonLetterCount = 0;
        String lowerText = text.toLowerCase();
        
        for (char c : commonLetters.toCharArray()) {
            commonLetterCount += countOccurrences(lowerText, c);
        }
        
        double commonLetterRatio = (double) commonLetterCount / text.length();

        // Last check: see if the word "the" shows up.
        boolean containsThe;
        containsThe = text.toLowerCase().contains("the");
        
        // English text typically has spaces making up 15-20% of characters
        // and common letters making up about 70% of all letters
        return spaceRatio > 0.1 && commonLetterRatio > 0.4 && containsThe;
    }
    
    /**
     * Count occurrences of a specific character in a string
     * @param text The text to search in
     * @param c The character to count
     * @return The number of occurrences
     */
    private static int countOccurrences(String text, char c) {
        int count = 0;
        for (int i = 0; i < text.length(); i++) {
            if (text.charAt(i) == c) {
                count++;
            }
        }
        return count;
    }
}
