import java.math.BigInteger;

public class Lychrel {
	public static void main(String[] args) {

		//System.out.println( countIterations(47)  );
		//System.out.println( isLychrel(47)        );
		//System.out.println( countIterations(349) );
		//System.out.println( isLychrel(349)		 );
		//System.out.println( countIterations(196) );
		//System.out.println( isLychrel(196)		 );
		int count = 0;
		for(int i=1; i<10000; i++){ 
			if(isLychrel(i)) { 
				count++;
			}
		}
		System.out.println(count);
	}

	/** Boolean: is n a Lychresl number (never reaches a palindrome)? */
	public static boolean isLychrel(int n) { 
		int it = countIterations(n);
		return it==51;
	}

	/** Count the number of iterations (steps) it takes to get to a palindrome.
	 * One step: 47 + 74 = 121
	 *
	 * Three steps: 
	 * 349 + 943 = 1292
	 * 1292 + 2921 = 4213
	 * 4213 + 3124 = 7337
	 */
	public static int countIterations(int n) {
		if(n>10000) { 
			throw new IllegalArgumentException("This method operates on the assumption that n < 10,000.");
		}

		// Even Palindromes can be Lychrel numbers, 
		// so do at least one step.
		//
		// steps is what we return.
		int steps = 0;
		final int MAXSTEPS = 50;

		BigInteger b = new BigInteger(Integer.toString(n));
		BigInteger r = new BigInteger(reverse(b.toString()));
		BigInteger s = b.add(r);
		steps++;

		while( !isPalindrome(s.toString()) && steps <= MAXSTEPS ) { 
			b = s;
			r = new BigInteger(reverse(b.toString()));
			s = b.add(r);
			steps++;
		}

		// This little twist of logic ensures that if there are exactly 50 iterations,
		// we don't accidentally classify it as a Lychrel number.
		if(!isPalindrome(s.toString())) {
			return MAXSTEPS+1;
		} else {
			return steps;
		}
	}




	/** Reverse a string. */
	public static String reverse(String in){
		char[] inr = new char[in.length()];
		for(int i=0; i<inr.length; i++) { 
			inr[i] = in.charAt(inr.length-i-1);
		}
		return new String(inr);
	}

	/** Check if string is palindrome. */
	public static boolean isPalindrome(String s) { 
		return s.equals(reverse(s));
	}




}
