import java.math.BigInteger;
import java.util.Collections;
import java.util.Arrays;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;

public class Problem062 {
	public static void main(String[] args) { 
		solve();
	}

	/** Set the problem size and solve it. */
	public static void solve() { 
		System.out.println(smallestCubePermutes(5));
	}

	/** Find the smallest cube that permutes with nperms other cubes. */
	public static BigInteger smallestCubePermutes(int nperms) { 
		/*
		 * The hash map is central to doing this fast.
		 * We compute a round of 10^{N-1} to 10^N perfect cubes,
		 * then sort their digits.
		 * This is the hash map key, and we store the resulting cube
		 * in a list so that we can go back and find the minimum cube later.
		 */

		Map<String,List<BigInteger>> m = new HashMap<String,List<BigInteger>>();

		int MAX = 10000;
		int ceil = 10;
		while(ceil<MAX) { 
			// Consider another order of magnitude
			ceil *= 10;
			for(int p=ceil/10; p<ceil; p++) { 

				BigInteger cube = BigInteger.ONE;
				// Cube it
				for(int j=0; j<3; j++) { 
					cube = cube.multiply( BigInteger.valueOf(p) );
				}

				// This is a bit messy.
				char[] cubeArr = cube.toString().toCharArray();
				Arrays.sort(cubeArr);
				String sortedCubeString = new String(cubeArr);

				if(m.keySet().contains(sortedCubeString)) {
					// If this permutation already exists in the map,
					// add this cube to the list and check if it's the nperms^th case.
					List<BigInteger> list = m.get(sortedCubeString);
					list.add(cube);
					if(list.size()==nperms) {
						// Someone in list is a winner.
						return Collections.min(list);
					}
				} else {
					// Start a new list
					List<BigInteger> list = new ArrayList<BigInteger>();
					list.add(cube);
					m.put(sortedCubeString,list);
				}
			}
		}
		System.out.println("Search for a cube that permutes "+nperms+" times failed. Search stopped at "+MAX+".");
		return null;

	}

}