import java.math.MathContext;
import java.math.RoundingMode;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Arrays;

/** Continued fraction representation.
 *
 * This program finds the sequence of integers
 * that show up in the continued fraction
 * representation of a square root.
 */
public class ContinuedFraction {

	public static void main(String[] args) { 
		test5();
	}

	public static void test5() { 
		int D; long x;
		D = 991;
		//for(int j=14; j<17; j++) { 
		//	long[] results = convergents(D,j);
		//	System.out.println("j = "+j+": "+results[0]+"/"+results[1]);
		//}
		System.out.println(continuedFractionSqrtBig(D));
	}

	public static void test4() { 
		int D; long x;
		D = 977;
		//for(int j=14; j<17; j++) { 
		//	long[] results = convergents(D,j);
		//	System.out.println("j = "+j+": "+results[0]+"/"+results[1]);
		//}
		System.out.println(continuedFractionSqrtBig(D));
	}

	public static void test3() { 
		int D; long x;
		D = 151;
		for(int j=10; j<24; j++) { 
			long[] results = convergents(D,j);
			System.out.println("j = "+j+": "+results[0]+"/"+results[1]);
		}
		System.out.println(continuedFractionSqrtBig(D));
	}

	public static void test2() { 
		int D; long x;
		D = 29;
		for(int j=5; j<10; j++) { 
			long[] results = convergents(D,j);
			System.out.println("j = "+j+": "+results[0]+"/"+results[1]);
		}
	}

	public static void test() {
		System.out.println("Testing sqrt(14):");
		System.out.println(continuedFractionSqrt(14));
		System.out.println("Should be [3, 1, 2, 1, 6].");
		System.out.println();


		System.out.println("Testing sqrt(29):");
		System.out.println(continuedFractionSqrt(29));
		System.out.println();


		long[] results;
		System.out.println("Testing convergents of sqrt(14):");
		results = convergents(14,3);
		System.out.println(results[0]+"/"+results[1]);
		System.out.println("Should be 15/4.");
		System.out.println();

		for(int i=0; i<10; i++) { 
			results = convergents(14,i);
			System.out.println("i = "+i+": "+results[0]+"/"+results[1]);
		}
		System.out.println();

		System.out.println("Testing convergents of sqrt(29):");
		for(int i=0; i<10; i++) { 
			results = convergents(29,i);
			System.out.println("i = "+i+": "+results[0]+"/"+results[1]);
		}
		System.out.println();


		int D;
		long x;
		D = 14;
		System.out.println("Testing Pell equation solution x^2 - D y^2 = 1, D = "+D);
		x = solvePell(D);
		System.out.println("D = "+D+", x = "+x);
		System.out.println("Should be 15.");
		System.out.println();

		D = 13;
		System.out.println("Testing Pell equation solution x^2 - D y^2 = 1, D = "+D);
		x = solvePell(D);
		System.out.println("D = "+D+", x = "+x);
		System.out.println("Should be 649.");
		System.out.println();

		D = 28;
		System.out.println("Testing Pell equation solution x^2 - D y^2 = 1, D = "+D);
		x = solvePell(D);
		System.out.println("D = "+D+", x = "+x);
		System.out.println();

		System.out.println("Duplicating Project Euler Problem 66 Example:");
		int[] ds = {2, 3, 5, 6, 7};
		for(int i=0; i<ds.length; i++) { 
			int j = ds[i];
			System.out.println("D = "+j+", x = "+solvePell(j));
		}
		System.out.println();
	}


	public static boolean isPerfectSquare(int x) { 
		int s = (int)(Math.round(Math.sqrt(x)));
		return x==(s*s);
	}

	/** 
	 * Solve the Pell equation,
	 * x^2 - D y^2 = 1.
	 * 
	 * This requires finding the continued fraction
	 * representation of D, and converting it into
	 * its convergents Pn and Qn, where n is...?
	 *
	 * Zeroth integer being added to the continued fraction?
	 *
	 * This returns x only.
	 */
	public static long solvePell(int D) {
		if(isPerfectSquare(D)) {
			return -1;
		}

		final int MAXITER = 300; // adjust this if you anticipate bigger fractions.

		List<Integer> cf = continuedFractionSqrtBig(D);

		long[] solutions = new long[2];
		int nterms = 1;
		while(!solvesPell(solutions[0],solutions[1],D) && nterms<MAXITER) { 
			solutions = convergents(D, nterms);
			nterms++;
		}
		return solutions[0];
	}

	/** Checks if this combination of x, y solves the Pell equation
	 * for the given value of D. */
	public static boolean solvesPell(long x, long y, int D) { 
		return (x*x - D*y*y == 1);
	}
		

	/** 
	 * Compute the convergents (rational representation of
	 * continued fraction).
	 *
	 * This uses the recurrence relation:
	 *
	 * P_n     a_n P_n-1  + P_n-2
	 * ---- = -----------------
	 *  Q_n    a_n Q_n-1  + Q_n-2
	 */
	public static long[] convergents(int n, int nterms) {
		long[] convergents = new long[2];

		List<Integer> cfrepr = continuedFractionSqrtBig(n);

		// Initial values for convergent recurrence relation
		long Pnm2 = 0; // P_{n-2}
		long Pnm1 = 1;
		long Qnm2 = 1;
		long Qnm1 = 0;
		long P = 0;
		long Q = 0;

		// Term 0 is the constant value a0.
		int accessindex = 0;
		for(int i=0; i<=nterms; i++) { 
			int an = cfrepr.get(accessindex);

			P = an * Pnm1 + Pnm2;
			Q = an * Qnm1 + Qnm2;

			Pnm2 = Pnm1;
			Pnm1 = P;

			Qnm2 = Qnm1;
			Qnm1 = Q;

			if(accessindex+1>=cfrepr.size()) { 
				// Ensure we keep repeating the sequence
				// if the sequence has fewer terms than
				// the user asks for.
				// This allows us to get really good
				// approximations for numbers.
				// This only works because the sequence
				// is palindromic.
				accessindex = 1;
			} else {
				accessindex++;
			}
		}

		convergents[0] = P;
		convergents[1] = Q;

		return convergents;
	}

	/** Find the floor of a double. */
	public static int floor(double j) {
		return (int)(Math.floor(j));
	}


	/** Find the (shorter than 10) continued fraction sequence for sqrt(n). 
	 * This returns a list of integers, [a0, a1, a2, ...]
	 */
	public static List<Integer> continuedFractionSqrt(int n) {
		int niters = 100; // handbrake

		int ai = 0;
		double val = 0;
		double remainder = 0;
		List<Integer> coeffs = new ArrayList<Integer>();

		// Fencepost
		remainder = 1.0/Math.sqrt(n);

		//System.out.println("val\t\tai\t\tremainder");
		for(int i=0; i<niters; i++) {
			val = 1.0/remainder;
			ai = floor(val);
			remainder = val - ai;
			coeffs.add(ai);
			//System.out.println(val+"\t\t"+ai+"\t\t"+remainder);
			if(coeffs.get(i) == 2*coeffs.get(0)) {
				break;
			}
		}
		return coeffs;
	}

	/** Find the (really long) continued fraction sequence for sqrt(n). 
	 * The BigDouble is needed if you have more than about 10 terms,
	 * since the very very tiny rounded portion of the remainder 
	 * becomes important for longer sequences. 
	 */
	public static List<Integer> continuedFractionSqrtBig(int n) {
		int niters = 100; // handbrake
		int SCALE = 100; // decimal places

		int ai = 0;

		List<Integer> coeffs = new ArrayList<Integer>();

		MathContext MATH_CTX = new MathContext(SCALE, RoundingMode.FLOOR);//HALF_UP);

		BigDecimal val = new BigDecimal(0, MATH_CTX);

		// Fencepost
		// NOTE: this truncates the decimal before it converts to BigDecimal, so it does not solve the problem.
		//BigDecimal remainder = new BigDecimal( 1.0/Math.sqrt(n) ,MATH_CTX ); 
		//
		// Instead, we have to implement an arbitrary-precision square root routine,
		// and take the square root of 1/n. 
		//
		// NOTE: this STILL truncates the decimals of 1/n before it converts to BigDecimal, so it does not solve the problem.
		//BigDecimal remainder = sqrtBig(BigDecimal.valueOf(1.0/n),SCALE);
		//
		BigDecimal remainder = BigDecimal.ONE;
		remainder = remainder.divide( BigDecimal.valueOf(n), MATH_CTX);
		remainder = sqrtBig(remainder, SCALE);


		for(int i=0; i<niters; i++) {

			val = BigDecimal.ONE;
			val = val.divide(remainder, MATH_CTX);
			ai = floor(val.doubleValue());
			remainder = val.subtract( BigDecimal.valueOf(ai) );
			coeffs.add(ai);
			if(coeffs.get(i) == 2*coeffs.get(0)) {
				break;
			}
		}
		return coeffs;
	}



	/** Find the square root of a BigDecimal to precision SCALE. */
	public static BigDecimal sqrtBig(BigDecimal A, final int SCALE) {
		BigDecimal x0 = new BigDecimal("0");
    	BigDecimal x1 = new BigDecimal(Math.sqrt(A.doubleValue()));
    	while (!x0.equals(x1)) {
    	    x0 = x1;
    	    x1 = A.divide(x0, SCALE, RoundingMode.HALF_UP);
    	    x1 = x1.add(x0);
    	    x1 = x1.divide(BigDecimal.valueOf(2), SCALE, RoundingMode.HALF_UP);

    	}
		return x1;
	}

}