import java.util.*;

/** 
 * Find the smallest number with 2^500500 divisors.
 *
 * This program builds from the ground up.
 * It relies on the procedure of multiplying
 * successively larger numbers, and uses a particular
 * procedure for choosing numbers in order to keep
 * the final product to a minimum.
 *
 * 2^500500 divisors means we choose 500500 factors
 * from our special table.
 *
 * Table should be sqrt(500500) x sqrt(500500) 
 * or 708 x 708
 *
 *
 *
 */
public class LotsaDivisors {


	//////////////////////////////////////////////////////
	// need to implement mod 500500507 with BigInteger
	//////////////////////////////////////////////////////

	public static final long POWER = 16;

	public static void main(String[] args) { 
		problem500();
	}


	public static void testSieve() { 

		List<Long> primes = primeSieveN(100);

		if(primes.get(0)!=2) { throw new AssertionError(); }
		if(primes.get(1)!=3) { throw new AssertionError(); }
		if(primes.get(2)!=5) { throw new AssertionError(); }

		if(primes.get(97)!=509) { throw new AssertionError(); }
		if(primes.get(98)!=523) { throw new AssertionError(); }
		if(primes.get(99)!=541) { throw new AssertionError(); }

		System.out.println("prime sieve working ok.");

	}


	public static void problem500() { 
		int m = (int)( 5 * Math.ceil(Math.sqrt(POWER)) );
		int b = (int)( 1.5 * Math.ceil(  Math.log(POWER)/Math.log(2) ) );
		TreeSet<Long> list = makeSet(m,b);

		long product = 1;
		int c = 0;
		for(Long r : list) { 
			if(c >= POWER) { 
				break;
			}
			product *= r;
			c++;
		}
		
		System.out.println("Smallest number with 2^"+POWER+" factors = "+product);
	}

	/** Make a sorted list of integers to choose from.
	 * This picks from a table with columns n, n^2, n^3, ..., n^log(POWER)
	 * and rows M = 1 ... sqrt(POWER)
	 * M rows, B columns. */
	public static TreeSet<Long> makeSet(int m, int b) {
		List<Long> primes = primeSieveN(m);
		TreeSet<Long> special = new TreeSet<Long>();
		for(int j=0; j<b; j++) {
			int e = (int)(Math.pow(2,j));
			for(int i=0; i<m; i++) { 
				long p = primes.get(i);
				special.add( (long)Math.pow(p,e) );
			}
		}
		return special;
	}




	/** Find n prime numbers.
	 */
	public static List<Long> primeSieveN(int n) { 

		// estimate of the nth prime number:
		// p(n) ~ n log n + n log log n
		// 
		// First term was the estimate used by Gauss.
		int approx_p = 20*(int)Math.round(n*Math.log(n) + n*Math.log(Math.log(n)));

        // initially assume all integers are prime 
        boolean[] isPrime = new boolean[approx_p+1];
        for (int i = 2; i <= approx_p; i++) {
            isPrime[i] = true;
        }

        // mark non-primes using Sieve of Eratosthenes
        for (int factor = 2; factor <= approx_p; factor++) {

            // if factor is prime, then mark multiples of factor as nonprime
            // suffices to consider mutiples factor, factor+1, ...,  n/factor
            if (isPrime[factor]) {
                for (int j = factor; factor*j <= approx_p; j++) {
                    isPrime[factor*j] = false;
                }
            }
        }

		// Add primes to list
		ArrayList<Long> primes = new ArrayList<Long>();
		int nprimes = 0;
		int k = 2;
		while( nprimes < n ) { 
			if(isPrime[k]) { 
				primes.add( new Long(k) );
				nprimes++;
			}
			k++;
		}
		return primes;
	}


}
