import java.util.Scanner;
import java.io.File;
import java.io.PrintStream;
import java.io.FileNotFoundException;

/**
 * Find the number of integers with exactly eight divisors
 * below 10^12.
 *
 * f(100) = 10
 * f(1000) = 180
 * f(1000000) = 224427
 */

public class Problem501 { 

	public static final long MAX = (long)(1000000);

	public static void main(String[] args) { 
		solve();
	}

	public static void solve() { 

		long eightFactors = 0;

		//////////////////////////////////////////////
		// Generate primes of the form:
		// pX^7
		// 
		// Or, just add as many as are less than MAX^(1/7)

		// This better be an int.
		int seventhRootMax = (int)(Math.floor(Math.pow(MAX,1.0/7.0)));
		PrimeGeneratorLong pgX = new PrimeGeneratorLong(seventhRootMax);

		while(pgX.hasNext()) { 
			pgX.next();
			eightFactors++;
		}

		System.out.println("Finished with first part. On to second part...");

		//////////////////////////////////////////////
		// Generate primes of the form:
		// pA^3 * pB

		PrimeGeneratorLong pgA = new PrimeGeneratorLong(MAX/8);
		PrimeGeneratorLong pgB = new PrimeGeneratorLong(MAX/8);

		int cubeRootMax = (int)(Math.ceil(Math.pow(MAX,1.0/3.0)));

		long pA, pB, prodA, prodB;
		boolean amaxed, bmaxed;
		while(pgA.hasNext()) { 
			pA = pgA.next();
			if(pA*pA*pA>MAX) { 
				break;
			}
			pgB.reset();
			while(pgB.hasNext()) { 
				pB = pgB.next();
				if(pB<=pA) {
					continue;
				}

				amaxed = false;
				if( pA<cubeRootMax ) {
					prodA = pA*pA*pA*pB;
					if(prodA<MAX) {
						eightFactors++;
					} else {
						amaxed = true;
					}
				}
				bmaxed = false;
				if( pB<cubeRootMax ) {
					prodB = pB*pB*pB*pA;
					if(prodB<MAX) {
						eightFactors++;
					} else {
						bmaxed = true;
					}
				}

				if(amaxed&&bmaxed) { break; }

			}
		}
		pgA = null;
		pgB = null;
		
		System.out.println("Finished with second part. On to third part...");

		//////////////////////////////////////////////
		// Generate primes of the form:
		// p1 * p2 * p3

		int sqRootMax = (int)(Math.ceil(Math.sqrt(MAX)));

		PrimeGeneratorLong pg1 = new PrimeGeneratorLong(MAX/6);
		PrimeGeneratorLong pg2 = new PrimeGeneratorLong(MAX/6);
		PrimeGeneratorLong pg3 = new PrimeGeneratorLong(MAX/6);

		int whatever = 0;

		long p1, p2, p3, product;
		while(pg1.hasNext()) { 
			p1 = pg1.next();
			if(p1*p1*p1>MAX) { 
				break;
			}
			pg2.reset();
			while(pg2.hasNext()) { 
				p2 = pg2.next();
				if( p1*p2*p2 > MAX || p1*p1*p2 > MAX ) { 
					break;
				}
				if(p2<=p1) {
					continue;
				}
				pg3.reset();
				while(pg3.hasNext()){ 
					p3 = pg3.next();
					if(p3<=p2) { 
						continue;
					}

					product = p1*p2*p3;
					whatever++;

					if(product<=MAX){
						eightFactors++;
						// Excellent interval for watching:
						//if(eightFactors%100000==0) { 
						//	System.out.println("eightFactors = "+eightFactors+" Product = " + product + " prime1 = "+p1+" prime2 = "+p2+" prime3 = "+p3);
						//}

					} else {
						// This will save our necks.
						break;
					}
				}
			}
		}
		pg1 = null;
		pg2 = null;
		pg3 = null;

		//////////////////////////////////////////////

		System.out.println("\nNumber of integers with exactly 8 factors:");
		System.out.println(eightFactors);

        try {
            PrintStream s = new PrintStream(new File("results.txt"));
		    s.println("\nNumber of integers with exactly 8 factors:");
            s.println(eightFactors);
        } catch (FileNotFoundException e) { 
            e.printStackTrace();
        }

	}
}

