from pprint import pprint

"""
Convex Castles
Project Euler 502

This program constructs all the convex castles that can be 
constructed on a W x H grid.

The procedure is to start with H Us, 1 R, and H Ds:
    U ... U R D ... D

Next, W-1 Rs are distributed among the 2(H-1) slots 
formed by pairs of Us or Ds, and the 1 middle slot
where there is already one R.

The castle variations are in another file.


Pseudocode:

main():
    convex_castles => construct_convex_castles()
    for castle in convex_castles:
        append castle to solutions
        variations = construct_variant_castles(castle)
        append variations to solutions


TODO: 

This program only constructs convex castles.

The program must be extended to insert strings of Us and Ds
between pairs of consecutive Rs. Otherwise, no odd H will work.

See dynamicprogram.py for working program to find castle variations.
"""

def main():
    W = 7
    H = 4
    convex_castles = construct_convex_castles(W, H)

    #pprint(convex_castles)
    print("Number of convex castles:")
    print("F(%d,%d) = %d"%( W, H, len(convex_castles)) )

    for convex_castle in convex_castles:
        print(convex_castle)

    ### for i,castle in enumerate(convex_castles):
    ###     castle_variations = construct_castle_variations(W, H, castle)
    ###     if(i is 0):
    ###         break



def construct_convex_castles(W, H):
    """
    Construct all convex castles.
    These have exactly H Us, H Ds, and W Rs.

    Start with the skeleton: H Us, H Ds

    U ... U D ... D

    Add one R separator

    U ... U R D ... D

    Then add Rs into available slots

    U_U_U_..._UR_D_D_..._D

    
    This constructs all convex castles.
    """
    convex_castles = set()

    base = ["U",]*H + ["R"] + ["D",]*H

    insert_r(base, 1, W, H, convex_castles)

    return convex_castles


def insert_r(castle, which_r, W, H, convex_castles):
    """
    Recursive method:
    Insert Rs into every position between Us and Ds.
    This is used to construct convex castles.
    Castle is a list.
    """
    global solutions_list
    if(which_r==W):
        # Base case:
        # No more r's left to place.
        # We stop at W because we placed one R already, 
        # so we placed W-1 more before stopping.

        # If we have an even # of blocks, add solution to list
        n_blocks = sum([1 if j is 'D' else 0 for j in castle])
        convex_castles.add("".join(castle))

        return convex_castles
    else:
        castle_orig = castle.copy()
        # try putting r in any available position:
        for i in range(1,len(castle_orig)):

            castle = castle_orig.copy()

            # Make a choice
            castle.insert(i,"R")

            # Explore the consequences
            convex_castles = insert_r(castle, which_r+1, W, H, convex_castles)

            # Unmake the choice
            del castle[i]

        return convex_castles



if __name__=="__main__":
    main()
