/**
 * Evaluate binomial coefficient using dynamic programming.
 */
public class BinomialDP {
	
	private long[][] bc;

	public BinomialDP(int n) { 
		// NOTE: 
		// n and k both begin at 0, not at 1.
		// Set bc[] size to n+1, then use bc.length,
		// instead of using n and possibly getting confused.

		// Initialize dynamic programming array
		// (this is basically a lower triangular matrix
		//  containing Pascal's Triangle)
		bc = new long[n+1][n+1];

		// Base cases: 
		// k choose 0 is 1
		// k choose k is 1
		for(int i=0; i<bc.length; i++) { 
			bc[i][0] = 1L;
			bc[i][i] = 1L;
		}

		// Fill in the rest
		for(int r=1; r<bc.length; r++) {
			for(int c=1; c<r; c++) { 
				bc[r][c] = bc[r-1][c-1] + bc[r-1][c];
			}
		}
	}

	public static void main(String[] args) { 
		int m = 26;
		BinomialDP b = new BinomialDP(m);
		for(int i=0; i<=m; i++) { 
			System.out.println("binomial("+m+","+i+") = "+b.binomial(m,i));
		}
	}

	public long binomial(int n, int k) { 
		return bc[n][k];
	}
}